@@ -933,9 +933,19 @@ void AvCallManager::setSelfAddress(const QHostAddress &addr)
 	d->rtpManager->setSelfAddress(addr);
 }
 
-void AvCallManager::setStunHost(const QString &host, int port)
+void AvCallManager::setStunBindService(const QString &host, int port)
 {
-	d->rtpManager->setStunHost(host, port);
+	d->rtpManager->setStunBindService(host, port);
+}
+
+void AvCallManager::setStunRelayUdpService(const QString &host, int port, const QString &user, const QString &pass)
+{
+	d->rtpManager->setStunRelayUdpService(host, port, user, pass);
+}
+
+void AvCallManager::setStunRelayTcpService(const QString &host, int port, const XMPP::AdvancedConnector::Proxy &proxy, const QString &user, const QString &pass)
+{
+	d->rtpManager->setStunRelayTcpService(host, port, proxy, user, pass);
 }
 
 void AvCallManager::setBasePort(int port)
