@@ -14,21 +14,24 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  */
 
 #ifndef DISCODLG_H
 #define DISCODLG_H
 
+#include <QDialog>
+
 #include "ui_disco.h"
+#include "xmpp_jid.h"
 
 class PsiAccount;
+class QString;
 
-#include "xmpp.h"
 using namespace XMPP;
 
-class DiscoDlg : public DiscoUI
+class DiscoDlg : public QDialog, public Ui::Disco
 {
 	Q_OBJECT
 public:
