@@ -14,22 +14,27 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  */
 
-#include"eventdb.h"
+#include "eventdb.h"
 
 #define FAKEDELAY 0
 
-#include<qfile.h>
-#include<qptrlist.h>
-#include<qtimer.h>
-#include<qtextstream.h>
-#include<qvaluevector.h>
-#include"common.h"
-#include"profiles.h"
+#include <QVector>
+#include <QFileInfo>
+#include <QDir>
+#include <QTimer>
+#include <QTextStream>
+#include <QDateTime>
 
+#include "common.h"
+#include "applicationinfo.h"
+#include "psievent.h"
+#include "jidutil.h"
+
+using namespace XMPP;
 
 //----------------------------------------------------------------------------
 // EDBItem
@@ -77,7 +82,7 @@ public:
 	Private() {}
 
 	EDB *edb;
-	EDBResult *r;
+	EDBResult r;
 	bool busy;
 	bool writeSuccess;
 	int listeningFor;
@@ -89,7 +94,6 @@ EDBHandle::EDBHandle(EDB *edb)
 {
 	d = new Private;
 	d->edb = edb;
-	d->r = 0;
 	d->busy = false;
 	d->writeSuccess = false;
 	d->listeningFor = -1;
@@ -102,7 +106,6 @@ EDBHandle::~EDBHandle()
 {
 	d->edb->unreg(this);
 
-	delete d->r;
 	delete d;
 }
 
@@ -127,6 +130,13 @@ void EDBHandle::get(const Jid &j, const QString &id, int direction, int len)
 	d->listeningFor = d->edb->op_get(j, id, direction, len);
 }
 
+void EDBHandle::getByDate(const Jid &j, QDateTime first, QDateTime last)
+{
+	d->busy = true;
+	d->lastRequestType = Read;
+	d->listeningFor = d->edb->op_getByDate(j, first, last);
+}
+
 void EDBHandle::find(const QString &str, const Jid &j, const QString &id, int direction)
 {
 	d->busy = true;
@@ -141,12 +151,19 @@ void EDBHandle::append(const Jid &j, PsiEvent *e)
 	d->listeningFor = d->edb->op_append(j, e);
 }
 
+void EDBHandle::erase(const Jid &j)
+{
+	d->busy = true;
+	d->lastRequestType = Erase;
+	d->listeningFor = d->edb->op_erase(j);
+}
+
 bool EDBHandle::busy() const
 {
 	return d->busy;
 }
 
-const EDBResult *EDBHandle::result() const
+const EDBResult EDBHandle::result() const
 {
 	return d->r;
 }
@@ -156,16 +173,10 @@ bool EDBHandle::writeSuccess() const
 	return d->writeSuccess;
 }
 
-void EDBHandle::edb_resultReady(EDBResult *r)
+void EDBHandle::edb_resultReady(EDBResult r)
 {
 	d->busy = false;
-	if(r == 0) {
-		delete d->r;
-		d->r = 0;
-	}
-	else {
-		d->r = r;
-	}
+	d->r = r;
 	d->listeningFor = -1;
 	finished();
 }
@@ -197,7 +208,7 @@ class EDB::Private
 public:
 	Private() {}
 
-	QPtrList<EDBHandle> list;
+	QList<EDBHandle*> list;
 	int reqid_base;
 };
 
@@ -209,7 +220,7 @@ EDB::EDB()
 
 EDB::~EDB()
 {
-	d->list.setAutoDelete(true);
+	qDeleteAll(d->list);
 	d->list.clear();
 	delete d;
 }
@@ -226,7 +237,7 @@ void EDB::reg(EDBHandle *h)
 
 void EDB::unreg(EDBHandle *h)
 {
-	d->list.removeRef(h);
+	d->list.removeAll(h);
 }
 
 int EDB::op_getLatest(const Jid &j, int len)
@@ -244,6 +255,11 @@ int EDB::op_get(const Jid &jid, const QString &id, int direction, int len)
 	return get(jid, id, direction, len);
 }
 
+int EDB::op_getByDate(const Jid &j, QDateTime first, QDateTime last)
+{
+	return getByDate(j, first, last);
+}
+
 int EDB::op_find(const QString &str, const Jid &j, const QString &id, int direction)
 {
 	return find(str, j, id, direction);
@@ -254,24 +270,26 @@ int EDB::op_append(const Jid &j, PsiEvent *e)
 	return append(j, e);
 }
 
-void EDB::resultReady(int req, EDBResult *r)
+int EDB::op_erase(const Jid &j)
+{
+	return erase(j);
+}
+
+void EDB::resultReady(int req, EDBResult r)
 {
 	// deliver
-	QPtrListIterator<EDBHandle> it(d->list);
-	for(EDBHandle *h; (h = it.current()); ++it) {
+	foreach(EDBHandle* h, d->list) {
 		if(h->listeningFor() == req) {
 			h->edb_resultReady(r);
 			return;
 		}
 	}
-	delete r;
 }
 
 void EDB::writeFinished(int req, bool b)
 {
 	// deliver
-	QPtrListIterator<EDBHandle> it(d->list);
-	for(EDBHandle *h; (h = it.current()); ++it) {
+	foreach(EDBHandle* h, d->list) {
 		if(h->listeningFor() == req) {
 			h->edb_writeFinished(b);
 			return;
@@ -293,6 +311,17 @@ struct item_file_req
 	int eventId;
 	QString findStr;
 	PsiEvent *event;
+
+	QDateTime first, last;
+	enum Type {
+		Type_getLatest = 0,
+		Type_getOldest,
+		Type_get,
+		Type_append,
+		Type_find,
+		Type_getByDate,
+		Type_erase
+	};
 };
 
 class EDBFlatFile::Private
@@ -300,8 +329,8 @@ class EDBFlatFile::Private
 public:
 	Private() {}
 
-	QPtrList<File> flist;
-	QPtrList<item_file_req> rlist;
+	QList<File*> flist;
+	QList<item_file_req*> rlist;
 };
 
 EDBFlatFile::EDBFlatFile()
@@ -312,8 +341,8 @@ EDBFlatFile::EDBFlatFile()
 
 EDBFlatFile::~EDBFlatFile()
 {
-	d->rlist.setAutoDelete(true);
-	d->flist.setAutoDelete(true);
+	qDeleteAll(d->rlist);
+	qDeleteAll(d->flist);
 	d->flist.clear();
 
 	delete d;
@@ -323,7 +352,7 @@ int EDBFlatFile::getLatest(const Jid &j, int len)
 {
 	item_file_req *r = new item_file_req;
 	r->j = j;
-	r->type = 0;
+	r->type = item_file_req::Type_getLatest;
 	r->len = len < 1 ? 1: len;
 	r->id = genUniqueId();
 	d->rlist.append(r);
@@ -336,7 +365,7 @@ int EDBFlatFile::getOldest(const Jid &j, int len)
 {
 	item_file_req *r = new item_file_req;
 	r->j = j;
-	r->type = 1;
+	r->type = item_file_req::Type_getOldest;
 	r->len = len < 1 ? 1: len;
 	r->id = genUniqueId();
 	d->rlist.append(r);
@@ -349,7 +378,7 @@ int EDBFlatFile::get(const Jid &j, const QString &id, int direction, int len)
 {
 	item_file_req *r = new item_file_req;
 	r->j = j;
-	r->type = 2;
+	r->type = item_file_req::Type_get;
 	r->len = len < 1 ? 1: len;
 	r->dir = direction;
 	r->eventId = id.toInt();
@@ -360,11 +389,27 @@ int EDBFlatFile::get(const Jid &j, const QString &id, int direction, int len)
 	return r->id;
 }
 
+
+int EDBFlatFile::getByDate(const XMPP::Jid &jid, QDateTime first, QDateTime last)
+{
+	item_file_req *r = new item_file_req;
+	r->j = jid;
+	r->type = item_file_req::Type_getByDate;
+	r->len = 1;
+	r->id = genUniqueId();
+	d->rlist.append(r);
+	r->first = first;
+	r->last = last;
+
+	QTimer::singleShot(FAKEDELAY, this, SLOT(performRequests()));
+	return r->id;
+}
+
 int EDBFlatFile::find(const QString &str, const Jid &j, const QString &id, int direction)
 {
 	item_file_req *r = new item_file_req;
 	r->j = j;
-	r->type = 4;
+	r->type = item_file_req::Type_find;
 	r->len = 1;
 	r->dir = direction;
 	r->findStr = str;
@@ -380,11 +425,26 @@ int EDBFlatFile::append(const Jid &j, PsiEvent *e)
 {
 	item_file_req *r = new item_file_req;
 	r->j = j;
-	r->type = 3;
-	if(e->type() == PsiEvent::Message)
-		r->event = new MessageEvent(*(MessageEvent *)e);
-	else if(e->type() == PsiEvent::Auth)
-		r->event = new AuthEvent(*(AuthEvent *)e);
+	r->type = item_file_req::Type_append;
+	r->event = e->copy();
+	if ( !r->event ) {
+		qWarning("EDBFlatFile::append(): Attempted to append incompatible type.");
+		delete r;
+		return 0;
+	}
+	r->id = genUniqueId();
+	d->rlist.append(r);
+
+	QTimer::singleShot(FAKEDELAY, this, SLOT(performRequests()));
+	return r->id;
+}
+
+int EDBFlatFile::erase(const Jid &j)
+{
+	item_file_req *r = new item_file_req;
+	r->j = j;
+	r->type = item_file_req::Type_erase;
+	r->event = 0;
 	r->id = genUniqueId();
 	d->rlist.append(r);
 
@@ -394,8 +454,7 @@ int EDBFlatFile::append(const Jid &j, PsiEvent *e)
 
 EDBFlatFile::File *EDBFlatFile::findFile(const Jid &j) const
 {
-	QPtrListIterator<File> it(d->flist);
-	for(File *i; (i = it.current()); ++it) {
+	foreach(File* i, d->flist) {
 		if(i->j.compare(j, false))
 			return i;
 	}
@@ -406,38 +465,65 @@ EDBFlatFile::File *EDBFlatFile::ensureFile(const Jid &j)
 {
 	File *i = findFile(j);
 	if(!i) {
-		i = new File(Jid(j.userHost()));
+		i = new File(Jid(j.bare()));
 		connect(i, SIGNAL(timeout()), SLOT(file_timeout()));
 		d->flist.append(i);
 	}
 	return i;
 }
 
+bool EDBFlatFile::deleteFile(const Jid &j)
+{
+	File *i = findFile(j);
+
+	QString fname;
+
+	if (i) {
+		fname = i->fname;
+		d->flist.removeAll(i);
+		delete i;
+	}
+	else {
+		fname = File::jidToFileName(j);
+	}
+
+	QFileInfo fi(fname);
+	if(fi.exists()) {
+		QDir dir = fi.dir();
+		return dir.remove(fi.fileName());
+	}
+	else
+		return true;
+}
+
 void EDBFlatFile::performRequests()
 {
 	if(d->rlist.isEmpty())
 		return;
 
-	item_file_req *r = d->rlist.first();
-	d->rlist.removeRef(r);
+	item_file_req *r = d->rlist.takeFirst();
 
 	File *f = ensureFile(r->j);
 	int type = r->type;
-	if(type >= 0 && type <= 2) {
+	if(type >= item_file_req::Type_getLatest && type <= item_file_req::Type_get) {
 		int id, direction;
 
-		if(type == 0) {
+		if(type == item_file_req::Type_getLatest) {
 			direction = Backward;
 			id = f->total()-1;
 		}
-		else if(type == 1) {
+		else if(type == item_file_req::Type_getOldest) {
 			direction = Forward;
 			id = 0;
 		}
-		else {
+		else if(type == item_file_req::Type_get) {
 			direction = r->dir;
 			id = r->eventId;
 		}
+		else {
+			qWarning("EDBFlatFile::performRequests(): Invalid type.");
+			return;
+		}
 
 		int len;
 		if(direction == Forward) {
@@ -453,8 +539,7 @@ void EDBFlatFile::performRequests()
 				len = r->len;
 		}
 
-		EDBResult *result = new EDBResult;
-		result->setAutoDelete(true);
+		EDBResult result;
 		for(int n = 0; n < len; ++n) {
 			PsiEvent *e = f->get(id);
 			if(e) {
@@ -463,8 +548,8 @@ void EDBFlatFile::performRequests()
 					prevId = QString::number(id-1);
 				if(id < f->total()-1)
 					nextId = QString::number(id+1);
-				EDBItem *ei = new EDBItem(e, QString::number(id), prevId, nextId);
-				result->append(ei);
+				EDBItemPtr ei = EDBItemPtr(new EDBItem(e, QString::number(id), prevId, nextId));
+				result.append(ei);
 			}
 
 			if(direction == Forward)
@@ -474,14 +559,13 @@ void EDBFlatFile::performRequests()
 		}
 		resultReady(r->id, result);
 	}
-	else if(type == 3) {
+	else if(type == item_file_req::Type_append) {
 		writeFinished(r->id, f->append(r->event));
 		delete r->event;
 	}
-	else if(type == 4) {
+	else if(type == item_file_req::Type_find) {
 		int id = r->eventId;
-		EDBResult *result = new EDBResult;
-		result->setAutoDelete(true);
+		EDBResult result;
 		while(1) {
 			PsiEvent *e = f->get(id);
 			if(!e)
@@ -493,15 +577,20 @@ void EDBFlatFile::performRequests()
 			if(id < f->total()-1)
 				nextId = QString::number(id+1);
 
+			bool matched = false;
 			if(e->type() == PsiEvent::Message) {
 				MessageEvent *me = (MessageEvent *)e;
 				const Message &m = me->message();
-				if(m.body().find(r->findStr, 0, false) != -1) {
-					EDBItem *ei = new EDBItem(e, QString::number(id), prevId, nextId);
-					result->append(ei);
-					break;
+				if(m.body().indexOf(r->findStr, 0, Qt::CaseInsensitive) != -1) {
+					EDBItemPtr ei = EDBItemPtr(new EDBItem(e, QString::number(id), prevId, nextId));
+					result.append(ei);
+					matched = true;
+					//commented line below to return ALL(instead of just first) messages that contain findStr
+					//break;
 				}
 			}
+			if (!matched)
+				delete e;
 
 			if(r->dir == Forward)
 				++id;
@@ -510,6 +599,41 @@ void EDBFlatFile::performRequests()
 		}
 		resultReady(r->id, result);
 	}
+	else if(type == item_file_req::Type_getByDate ) {
+		int id = 0;
+		EDBResult result;
+		for (int i=0; i < f->total(); ++i) {
+			PsiEvent *e = f->get(id);
+			if(!e)
+				continue;
+
+			QString prevId, nextId;
+			if(id > 0)
+				prevId = QString::number(id-1);
+			if(id < f->total()-1)
+				nextId = QString::number(id+1);
+
+			bool matched = false;
+			if(e->type() == PsiEvent::Message) {
+				MessageEvent *me = (MessageEvent *)e;
+				const Message &m = me->message();
+				if(m.timeStamp() > r->first && m.timeStamp() < r->last ) {
+					EDBItemPtr ei = EDBItemPtr(new EDBItem(e, QString::number(id), prevId, nextId));
+					result.append(ei);
+					matched = true;
+				}
+			}
+			if (!matched)
+				delete e;
+
+			++id;
+		}
+		resultReady(r->id, result);
+	}
+
+	else if(type == item_file_req::Type_erase) {
+		writeFinished(r->id, deleteFile(f->j));
+	}
 
 	delete r;
 }
@@ -517,8 +641,8 @@ void EDBFlatFile::performRequests()
 void EDBFlatFile::file_timeout()
 {
 	File *i = (File *)sender();
-	d->flist.remove(i);
-	delete i;
+	d->flist.removeAll(i);
+	i->deleteLater();
 }
 
 
@@ -530,12 +654,14 @@ class EDBFlatFile::File::Private
 public:
 	Private() {}
 
-	QValueVector<int> index;
+	QVector<quint64> index;
+	bool indexed;
 };
 
 EDBFlatFile::File::File(const Jid &_j)
 {
 	d = new Private;
+	d->indexed = false;
 
 	j = _j;
 	valid = false;
@@ -543,39 +669,10 @@ EDBFlatFile::File::File(const Jid &_j)
 	connect(t, SIGNAL(timeout()), SLOT(timer_timeout()));
 
 	//printf("[EDB opening -- %s]\n", j.full().latin1());
-	QString s = j.userHost();
-	fname = getHistoryDir() + "/" + qstrlower(jidEncode(s)) + ".history";
-	f.setName(fname);
-	valid = f.open(IO_ReadWrite);
-	if(valid) {
-		//printf(" file: %s\n", fname.latin1());
-		// build index
-		while(1) {
-			int at = f.at();
+	fname = jidToFileName(_j);
+	f.setFileName(fname);
+	valid = f.open(QIODevice::ReadWrite);
 
-			// locate a newline
-			bool found = false;
-			do {
-				int c = f.getch();
-				if(c == '\n') {
-					found = true;
-					break;
-				}
-			} while(!f.atEnd());
-
-			if(!found)
-				break;
-
-			int oldsize = d->index.size();
-			d->index.resize(oldsize+1);
-			d->index[oldsize] = at;
-		}
-	}
-	else {
-		//printf(" file: can't open\n");
-	}
-
-	//printf(" messages: %d\n\n", d->index.size());
 	touch();
 }
 
@@ -588,8 +685,57 @@ EDBFlatFile::File::~File()
 	delete d;
 }
 
+QString EDBFlatFile::File::jidToFileName(const XMPP::Jid &j)
+{
+	return ApplicationInfo::historyDir() + "/" + JIDUtil::encode(j.bare()).toLower() + ".history";
+}
+
+void EDBFlatFile::File::ensureIndex()
+{
+	if ( valid && !d->indexed ) {
+		if (f.isSequential()) {
+			qWarning("EDBFlatFile::File::ensureIndex(): Can't index sequential files.");
+			return;
+		}
+
+		f.reset(); // go to beginning
+		d->index.clear();
+
+		//printf(" file: %s\n", fname.latin1());
+		// build index
+		while(1) {
+			quint64 at = f.pos();
+
+			// locate a newline
+			bool found = false;
+			char c;
+			while (f.getChar(&c)) {
+				if (c == '\n') {
+					found = true;
+					break;
+				}
+			}
+
+			if(!found)
+				break;
+
+			int oldsize = d->index.size();
+			d->index.resize(oldsize+1);
+			d->index[oldsize] = at;
+		}
+
+		d->indexed = true;
+	}
+	else {
+		//printf(" file: can't open\n");
+	}
+
+	//printf(" messages: %d\n\n", d->index.size());
+}
+
 int EDBFlatFile::File::total() const
 {
+	((EDBFlatFile::File *)this)->ensureIndex();
 	return d->index.size();
 }
 
@@ -610,16 +756,16 @@ PsiEvent *EDBFlatFile::File::get(int id)
 	if(!valid)
 		return 0;
 
-	if(id < 0 || id > (int)d->index.size())
+	ensureIndex();
+	if(id < 0 || id >= (int)d->index.size())
 		return 0;
 
-	f.at(d->index[id]);
+	f.seek(d->index[id]);
 
 	QTextStream t;
 	t.setDevice(&f);
-	t.setEncoding(QTextStream::UnicodeUTF8);
+	t.setCodec("UTF-8");
 	QString line = t.readLine();
-	t.unsetDevice();
 
 	return lineToEvent(line);
 }
@@ -635,19 +781,20 @@ bool EDBFlatFile::File::append(PsiEvent *e)
 	if(line.isEmpty())
 		return false;
 
-	f.at(f.size());
-	int at = f.at();
+	f.seek(f.size());
+	quint64 at = f.pos();
 
 	QTextStream t;
 	t.setDevice(&f);
-	t.setEncoding(QTextStream::UnicodeUTF8);
+	t.setCodec("UTF-8");
 	t << line << endl;
-	t.unsetDevice();
 	f.flush();
 
-	int oldsize = d->index.size();
-	d->index.resize(oldsize+1);
-	d->index[oldsize] = at;
+	if ( d->indexed ) {
+		int oldsize = d->index.size();
+		d->index.resize(oldsize+1);
+		d->index[oldsize] = at;
+	}
 
 	return true;
 }
@@ -657,40 +804,40 @@ PsiEvent *EDBFlatFile::File::lineToEvent(const QString &line)
 	// -- read the line --
 	QString sTime, sType, sOrigin, sFlags, sText, sSubj, sUrl, sUrlDesc;
 	int x1, x2;
-	x1 = line.find('|') + 1;
+	x1 = line.indexOf('|') + 1;
 
-	x2 = line.find('|', x1);
+	x2 = line.indexOf('|', x1);
 	sTime = line.mid(x1, x2-x1);
 	x1 = x2 + 1;
 
-	x2 = line.find('|', x1);
+	x2 = line.indexOf('|', x1);
 	sType = line.mid(x1, x2-x1);
 	x1 = x2 + 1;
 
-	x2 = line.find('|', x1);
+	x2 = line.indexOf('|', x1);
 	sOrigin = line.mid(x1, x2-x1);
 	x1 = x2 + 1;
 
-	x2 = line.find('|', x1);
+	x2 = line.indexOf('|', x1);
 	sFlags = line.mid(x1, x2-x1);
 	x1 = x2 + 1;
 
 	// check for extra fields
 	if(sFlags[1] != '-') {
-		int subflags = hexChar2int(sFlags[1].latin1());
+		int subflags = QString(sFlags[1]).toInt(NULL,16);
 
 		// have subject?
 		if(subflags & 1) {
-			x2 = line.find('|', x1);
+			x2 = line.indexOf('|', x1);
 			sSubj = line.mid(x1, x2-x1);
 			x1 = x2 + 1;
 		}
 		// have url?
 		if(subflags & 2) {
-			x2 = line.find('|', x1);
+			x2 = line.indexOf('|', x1);
 			sUrl = line.mid(x1, x2-x1);
 			x1 = x2 + 1;
-			x2 = line.find('|', x1);
+			x2 = line.indexOf('|', x1);
 			sUrlDesc = line.mid(x1, x2-x1);
 			x1 = x2 + 1;
 		}
@@ -719,7 +866,7 @@ PsiEvent *EDBFlatFile::File::lineToEvent(const QString &line)
 		if(sFlags[0] == 'N')
 			m.setBody(logdecode(sText));
 		else
-			m.setBody(logdecode(QString::fromUtf8(sText)));
+			m.setBody(logdecode(QString::fromUtf8(sText.toLatin1())));
 		m.setSubject(logdecode(sSubj));
 
 		QString url = logdecode(sUrl);
@@ -755,8 +902,8 @@ PsiEvent *EDBFlatFile::File::lineToEvent(const QString &line)
 		ae->setTimeStamp(QDateTime::fromString(sTime, Qt::ISODate));
 		return ae;
 	}
-	else
-		return 0;
+
+	return NULL;
 }
 
 QString EDBFlatFile::File::eventToLine(PsiEvent *e)
@@ -787,7 +934,7 @@ QString EDBFlatFile::File::eventToLine(PsiEvent *e)
 		sFlags = "N---";
 
 		if(subflags != 0)
-			sFlags[1] = int2hexChar(subflags);
+			sFlags[1] = QString::number(subflags,16)[0];
 
 		//  | date | type | To/from | flags | text
 		QString line = "|" + sTime + "|" + sType + "|" + sOrigin + "|" + sFlags + "|";
@@ -827,6 +974,6 @@ QString EDBFlatFile::File::eventToLine(PsiEvent *e)
 
 		return line;
 	}
-	else
-		return "";
+	
+	return "";
 }
