@@ -1,70 +1,166 @@
 #include "opt_iconset.h"
 #include "common.h"
 #include "iconwidget.h"
+#include "applicationinfo.h"
+#include "psiiconset.h"
 #include "psicon.h"
+#include "psioptions.h"
 
-#include <qbuttongroup.h>
-#include <qwhatsthis.h>
-#include <qcheckbox.h>
-#include <qradiobutton.h>
-#include <qcombobox.h>
-#include <qlineedit.h>
-#include <qfileinfo.h>
-#include <qlistview.h>
-#include <qheader.h>
-#include <qapplication.h>
-#include <qthread.h>
-#include <qmutex.h>
-#include <qevent.h>
-#include <qprogressbar.h>
-#include <qcursor.h>
-#include <qpalette.h>
-#include <qtabwidget.h>
+#include <QDir>
+#include <QCheckBox>
+#include <QRadioButton>
+#include <QComboBox>
+#include <QLineEdit>
+#include <QFileInfo>
+#include <QApplication>
+#include <QThread>
+#include <QMutex>
+#include <QEvent>
+#include <QCursor>
+#include <QPalette>
+#include <QTabWidget>
+#include <QFont>
+#include <QTreeWidget>
 
-#include "opt_iconset_emo.h"
-#include "opt_iconset_system.h"
-#include "opt_iconset_roster.h"
-#include "ui_isdetails.h"
+#include "ui_opt_iconset_emo.h"
+#include "ui_opt_iconset_system.h"
+#include "ui_opt_iconset_roster.h"
+#include "ui_ui_isdetails.h"
 
-static void isDetails(const Iconset &is, QWidget *parent)
+class IconsetEmoUI : public QWidget, public Ui::IconsetEmo
 {
-	IconsetDetailsDlg *isd = new IconsetDetailsDlg(parent, "IconsetDetailsDlg", true);
-	isd->setIconset(is);
-	isd->exec();
-	delete isd;
-}
+public:
+	IconsetEmoUI() : QWidget() { setupUi(this); }
+};
 
-static QStringList dirs;
+class IconsetSystemUI : public QWidget, public Ui::IconsetSystem
+{
+public:
+	IconsetSystemUI() : QWidget() { setupUi(this); }
+};
+
+class IconsetRosterUI : public QWidget, public Ui::IconsetRoster
+{
+public:
+	IconsetRosterUI() : QWidget() { setupUi(this); }
+};
+
+class IconsetDetailsDlg : public QDialog, public Ui::IconsetDetailsDlg
+{
+public:
+	IconsetDetailsDlg(PsiCon *psicon, QWidget* parent, const char* name, bool modal)
+		: QDialog(parent)
+		, psi(psicon)
+	{
+		setAttribute(Qt::WA_DeleteOnClose);
+		setupUi(this);
+
+		psi->dialogRegister(this);
+
+		QStringList bold_labels;
+		bold_labels << "lb_name2";
+		bold_labels << "lb_version2";
+		bold_labels << "lb_date2";
+		bold_labels << "lb_home2";
+		bold_labels << "lb_desc2";
+		bold_labels << "lb_authors";
+
+		QList<QLabel *> labels = findChildren<QLabel *>();
+		foreach (QLabel *l, labels) {
+			if (bold_labels.contains(l->objectName())) {
+				QFont font = l->font();
+				font.setBold(true);
+				l->setFont(font);
+			}
+		}
+		
+		setObjectName(name);
+		setModal(modal);
+	}
+
+	~IconsetDetailsDlg()
+	{
+		psi->dialogUnregister(this);
+	}
+	
+	void setIconset( const Iconset &is ) {
+		setWindowTitle(windowTitle().arg(is.name()));
+		
+		lb_name->setText(is.name());
+		lb_version->setText(is.version());
+		
+		if ( is.description().isEmpty() ) {
+			lb_desc->hide();
+			lb_desc2->hide();
+		}
+		else
+			lb_desc->setText(is.description());
+		
+		if ( !is.homeUrl().isEmpty() ) {
+			lb_home->setTitle(is.homeUrl());
+			lb_home->setUrl(is.homeUrl());
+		}
+		else {
+			lb_home->hide();
+			lb_home2->hide();
+		}
+			
+		if ( is.creation().isEmpty() ) {
+			lb_date->hide();
+			lb_date2->hide();
+		}
+		else {
+			QDate date = QDate::fromString(is.creation(), Qt::ISODate);	
+			lb_date->setText(date.toString(Qt::LocalDate));
+		}
+		
+		if ( is.authors().count() == 0 ) {
+			ptv_authors->hide();
+			lb_authors->hide();
+		}
+		else {
+			QString authors;
+			QStringList::ConstIterator it = is.authors().begin();
+			for ( ; it != is.authors().end(); ++it) {
+				if ( !authors.isEmpty() )
+					authors += "<br><br>";
+				authors += *it;
+			}
+			ptv_authors->setText( "<qt><nobr>" + authors + "</nobr></qt>" );
+		}
+		
+		isd_iconset->setIconset(is);
+		
+		resize(sizeHint());
+	}
+
+private:
+	PsiCon *psi;
+};
+
+static void isDetails(const Iconset &is, QWidget *parent, PsiCon *psi)
+{
+	IconsetDetailsDlg *isd = new IconsetDetailsDlg(psi, parent, "IconsetDetailsDlg", false);
+	isd->setIconset(is);
+	isd->show();
+}
 
 static int countIconsets(QString addDir, QStringList excludeList)
 {
 	int count = 0;
 
-	QStringList::Iterator it = dirs.begin();
-	for ( ; it != dirs.end(); ++it) {
-		QString fileName = *it + "/iconsets" + addDir;
-		QDir dir (fileName);
+	foreach (const QString &dataDir,  ApplicationInfo::dataDirs()) {
+		QDir dir (dataDir + "/iconsets" + addDir);
 
-		QStringList list = dir.entryList("*");
-		QStringList::Iterator it2 = list.begin();
-		for ( ; it2 != list.end(); ++it2) {
-			if ( *it2 == "." || *it2 == ".." )
-				continue;
-
-			bool found = false;
-			QStringList::Iterator it3 = excludeList.begin();
-			for ( ; it3 != excludeList.end(); ++it3) {
-				if ( *it2 == *it3 ) {
-					found = true;
-					break;
-				}
-			}
-
-			if ( found )
+		foreach (const QFileInfo &fi,
+				 dir.entryInfoList(QDir::NoDotAndDotDot | QDir::AllEntries))
+		{
+			QString iconsetId = fi.absoluteFilePath().section('/', -2);
+			if ( excludeList.contains(iconsetId) || !Iconset::isSourceAllowed(fi))
 				continue;
 
 			count++;
-			excludeList << *it2;
+			excludeList << iconsetId;
 		}
 	}
 
@@ -79,15 +175,15 @@ class IconsetLoadEvent : public QEvent
 {
 public:
 	IconsetLoadEvent(IconsetLoadThread *par, Iconset *i)
-	: QEvent(QEvent::User)
+		: QEvent(QEvent::User)
+		, p(par)
+		, is(i)
 	{
-		p  = par;
-		is = i;
 	}
 
 	IconsetLoadThread *thread() const { return p; }
 
-	// if iconset() is '0' then it means that iconset wans't loaded successfully
+	// if iconset() is '0' then it means that iconset wasn't loaded successfully
 	Iconset *iconset() const { return is; }
 
 private:
@@ -103,7 +199,7 @@ class IconsetFinishEvent : public QEvent
 {
 public:
 	IconsetFinishEvent()
-	: QEvent( (QEvent::Type)(QEvent::User + 1) )
+		: QEvent( (QEvent::Type)(QEvent::User + 1) )
 	{
 	}
 };
@@ -117,9 +213,9 @@ class IconsetLoadThreadDestroyEvent : public QEvent
 {
 public:
 	IconsetLoadThreadDestroyEvent(QThread *t)
-	: QEvent( (QEvent::Type)(QEvent::User + 2) )
+		: QEvent( (QEvent::Type)(QEvent::User + 2) )
+		, thread(t)
 	{
-		thread = t;
 	}
 
 	~IconsetLoadThreadDestroyEvent()
@@ -155,10 +251,10 @@ private:
 };
 
 IconsetLoadThread::IconsetLoadThread(QObject *p, QString path)
+	: cancelled(false)
+	, parent(p)
+	, addPath(path)
 {
-	cancelled = false;
-	parent  = p;
-	addPath = path;
 }
 
 void IconsetLoadThread::excludeIconsets(QStringList l)
@@ -185,18 +281,16 @@ void IconsetLoadThread::postEvent(QEvent *e)
 void IconsetLoadThread::run()
 {
 	threadMutex.lock();
-	QStringList dirs = ::dirs;
+	QStringList dirs = ApplicationInfo::dataDirs();
 	threadMutex.unlock();
 
-	QStringList::Iterator it = dirs.begin();
-	for ( ; it != dirs.end(); ++it) {
-		QString fileName = *it + "/iconsets" + addPath;
-		QDir dir (fileName);
-
-		QStringList list = dir.entryList("*");
-		QStringList::Iterator it2 = list.begin();
-		for ( ; it2 != list.end(); ++it2) {
-			if ( *it2 == "." || *it2 == ".." )
+	QStringList failedList;
+	foreach (const QString &dataDir, dirs) {
+		QDir dir(dataDir + "/iconsets" + addPath);
+		foreach (const QFileInfo &iconsetFI,
+				 dir.entryInfoList(QDir::NoDotAndDotDot | QDir::AllEntries))
+		{
+			if (!Iconset::isSourceAllowed(iconsetFI))
 				continue;
 
 			threadCancelled.lock();
@@ -206,54 +300,30 @@ void IconsetLoadThread::run()
 			if ( cancel )
 				goto getout;
 
-			bool found = false;
-			QStringList::Iterator it3 = excludeList.begin();
-			for ( ; it3 != excludeList.end(); ++it3) {
-				if ( *it2 == *it3 ) {
-					found = true;
-					break;
-				}
-			}
-
-			if ( found )
+			QString iconsetId = iconsetFI.absoluteFilePath().section('/', -2);
+			if (excludeList.contains(iconsetId))
 				continue;
 
-			IconsetLoadEvent *event = 0;
 			Iconset *is = new Iconset;
 
-			if ( is->load (fileName + "/" + *it2) ) {
-				excludeList << *it2;
+			if ( is->load (iconsetFI.absoluteFilePath()) ) {
+				failedList.removeOne(is->id());
+				excludeList << is->id();
 
 				// don't forget to delete iconset in ::event()!
-				event = new IconsetLoadEvent(this, is);
+				postEvent(new IconsetLoadEvent(this, is));
 			}
 			else {
 				delete is;
-				event = new IconsetLoadEvent(this, 0);
-
-				// without excluding corrupted iconset,
-				// counter will go nuts! read more comments
-				// about that...
-				excludeList << *it2;
-				// TODO: there is possibility,
-				// that there's a bunch of same-named
-				// iconsets, and some of them are corrupted.
-				// It is possible to write a hack that
-				// loads iconset even in that case.
-
-				// logic:
-				//   tried to load iconset --> unable to load
-				//   checking if there's same-named iconsets in
-				//   other directories
-				//   emit IconsetLoadEvent() only on success
-				//   or when last corrupted iconset was unable
-				//   to load
+				failedList << iconsetId;
 			}
-
-			postEvent(event);
 		}
 	}
 
+	for(int i = 0; i < failedList.size(); i++) {
+		postEvent(new IconsetLoadEvent(this, 0));
+	}
+
 getout:
 	postEvent(new IconsetFinishEvent());
 	QApplication::postEvent(qApp, new IconsetLoadThreadDestroyEvent(this)); // self destruct
@@ -264,16 +334,10 @@ getout:
 //----------------------------------------------------------------------------
 
 OptionsTabIconsetSystem::OptionsTabIconsetSystem(QObject *parent)
-: OptionsTab(parent, "iconset_system", "", tr("System Iconset"), tr("Select the system iconset"), "psi/info")
+	: OptionsTab(parent, "iconset_system", "", tr("System Icons"), tr("Select the system iconset"))
+	, w(0)
+	, thread(0)
 {
-	w = 0;
-	thread = 0;
-
-	if ( dirs.isEmpty() ) {
-		dirs << ".";
-		dirs << g.pathHome;
-		dirs << g.pathBase;
-	}
 }
 
 OptionsTabIconsetSystem::~OptionsTabIconsetSystem()
@@ -296,7 +360,7 @@ QWidget *OptionsTabIconsetSystem::widget()
 	return w;
 }
 
-void OptionsTabIconsetSystem::applyOptions(Options *opt)
+void OptionsTabIconsetSystem::applyOptions()
 {
 	if ( !w || thread )
 		return;
@@ -305,32 +369,26 @@ void OptionsTabIconsetSystem::applyOptions(Options *opt)
 	const Iconset *is = d->iss_system->iconset();
 	if ( is ) {
 		QFileInfo fi( is->fileName() );
-		opt->systemIconset = fi.fileName();
+		PsiOptions::instance()->setOption("options.iconsets.system", fi.fileName());
 	}
 }
 
-void OptionsTabIconsetSystem::restoreOptions(const Options *opt)
+void OptionsTabIconsetSystem::restoreOptions()
 {
 	if ( !w || thread )
 		return;
 
-	o = (Options *)opt;
 	IconsetSystemUI *d = (IconsetSystemUI *)w;
 	d->iss_system->clear();
 	QStringList loaded;
 
-	d->setCursor(WaitCursor);
+	d->setCursor(Qt::WaitCursor);
 
-	QPalette customPal = d->palette();
-	customPal.setDisabled(customPal.inactive());
 	d->iss_system->setEnabled(false);
-	d->iss_system->setPalette(customPal);
-
 	d->pb_sysDetails->setEnabled(false);
-	d->pb_sysDetails->setPalette(customPal);
 
 	d->progress->show();
-	d->progress->setProgress( 0 );
+	d->progress->setValue( 0 );
 
 	numIconsets = countIconsets("/system", loaded);
 	iconsetsLoaded = 0;
@@ -352,16 +410,16 @@ bool OptionsTabIconsetSystem::event(QEvent *e)
 
 		if ( !numIconsets )
 			numIconsets = 1;
-		d->progress->setProgress( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
+		d->progress->setValue( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
 
 		Iconset *i = le->iconset();
 		if ( i ) {
-			is->stripFirstAnimFrame(i);
+			PsiIconset::instance()->stripFirstAnimFrame(i);
 			d->iss_system->insert(*i);
 
 			QFileInfo fi( i->fileName() );
-			if ( fi.fileName() == o->systemIconset )
-				d->iss_system->setSelected(d->iss_system->count()-1, true);
+			if ( fi.fileName() == PsiOptions::instance()->getOption("options.iconsets.system").toString() )
+				d->iss_system->setItemSelected(d->iss_system->lastItem(), true);
 
 			delete i;
 		}
@@ -370,12 +428,9 @@ bool OptionsTabIconsetSystem::event(QEvent *e)
 	}
 	else if ( e->type() == QEvent::User + 1 ) { // finish event
 		d->iss_system->setEnabled(true);
-		d->iss_system->unsetPalette();
-
 		d->pb_sysDetails->setEnabled(true);
-		d->pb_sysDetails->unsetPalette();
 
-		connect(d->iss_system, SIGNAL(selectionChanged()), SIGNAL(dataChanged()));
+		connect(d->iss_system, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SIGNAL(dataChanged()));
 
 		d->progress->hide();
 
@@ -392,12 +447,14 @@ void OptionsTabIconsetSystem::previewIconset()
 {
 	IconsetSystemUI *d = (IconsetSystemUI *)w;
 	const Iconset *is = d->iss_system->iconset();
-	if ( is )
-		isDetails(*is, parentWidget);
+	if (is) {
+		isDetails(*is, parentWidget->parentWidget(), psi);
+	}
 }
 
-void OptionsTabIconsetSystem::setData(PsiCon *, QWidget *p)
+void OptionsTabIconsetSystem::setData(PsiCon *psicon, QWidget *p)
 {
+	psi = psicon;
 	parentWidget = p;
 }
 
@@ -417,10 +474,10 @@ void OptionsTabIconsetSystem::cancelThread()
 //----------------------------------------------------------------------------
 
 OptionsTabIconsetEmoticons::OptionsTabIconsetEmoticons(QObject *parent)
-: OptionsTab(parent, "iconset_emoticons", "", tr("Emoticons"), tr("Select your emoticon iconsets"), "psi/smile")
+	: OptionsTab(parent, "iconset_emoticons", "", tr("Emoticons"), tr("Select your emoticon iconsets"))
+	, w(0)
+	, thread(0)
 {
-	w = 0;
-	thread = 0;
 }
 
 OptionsTabIconsetEmoticons::~OptionsTabIconsetEmoticons()
@@ -442,7 +499,7 @@ QWidget *OptionsTabIconsetEmoticons::widget()
 	connect(d->pb_emoDown, SIGNAL(clicked()), SIGNAL(dataChanged()));
 	connect(d->pb_emoDetails, SIGNAL(clicked()), SLOT(previewIconset()));
 
-	QWhatsThis::add(d->ck_useEmoticons,
+	d->ck_useEmoticons->setWhatsThis(
 		tr("<P>Emoticons are short sequences of characters that are used to convey an emotion or idea.</P>"
 		"<P>Enable this option if you want Psi to replace common emoticons with a graphical image.</P>"
 		"<P>For example, <B>:-)</B> would be replaced by <icon name=\"psi/smile\"></P>"));
@@ -452,47 +509,44 @@ QWidget *OptionsTabIconsetEmoticons::widget()
 	return w;
 }
 
-void OptionsTabIconsetEmoticons::applyOptions(Options *opt)
+void OptionsTabIconsetEmoticons::applyOptions()
 {
 	if ( !w || thread )
 		return;
 
 	IconsetEmoUI *d = (IconsetEmoUI *)w;
-	opt->useEmoticons = d->ck_useEmoticons->isChecked();
+	PsiOptions::instance()->setOption("options.ui.emoticons.use-emoticons", d->ck_useEmoticons->isChecked());
 
-	opt->emoticons.clear();
-	IconWidgetItem *item = (IconWidgetItem *)d->iss_emoticons->firstItem();
-	while ( item ) {
-		if ( item->isSelected() ) {
+	QStringList list;
+	for (int row = 0; row < d->iss_emoticons->count(); row++) {
+		IconWidgetItem *item = (IconWidgetItem *)d->iss_emoticons->item(row);
+
+		if ( d->iss_emoticons->isItemSelected(item) ) {
 			const Iconset *is = item->iconset();
 			if ( is ) {
 				QFileInfo fi( is->fileName() );
-				opt->emoticons << fi.fileName();
+				list << fi.fileName();
 			}
 		}
-
-		item = (IconWidgetItem *)item->next();
 	}
-
+	PsiOptions::instance()->setOption("options.iconsets.emoticons", list);
 }
 
-void OptionsTabIconsetEmoticons::restoreOptions(const Options *opt)
+void OptionsTabIconsetEmoticons::restoreOptions()
 {
 	if ( !w || thread )
 		return;
 
 	IconsetEmoUI *d = (IconsetEmoUI *)w;
-	d->ck_useEmoticons->setChecked( opt->useEmoticons );
+	d->ck_useEmoticons->setChecked( PsiOptions::instance()->getOption("options.ui.emoticons.use-emoticons").toBool() );
 
 	// fill in the iconset view
 	d->iss_emoticons->clear();
 
 	{
-		QPtrListIterator<Iconset> it ( is->emoticons );
-		Iconset *is;
-		for ( ; (is = it.current()); ++it) {
+		foreach(Iconset* is, PsiIconset::instance()->emoticons) {
 			d->iss_emoticons->insert(*is);
-			d->iss_emoticons->setSelected(d->iss_emoticons->count()-1, true);
+			d->iss_emoticons->setItemSelected(d->iss_emoticons->lastItem(), true);
 		}
 	}
 
@@ -500,26 +554,18 @@ void OptionsTabIconsetEmoticons::restoreOptions(const Options *opt)
 	{
 		QStringList loaded;
 		{
-			QPtrListIterator<Iconset> it( is->emoticons );
-			Iconset *tmp;
-			for ( ; (tmp = it.current()); ++it) {
-				QFileInfo fi ( tmp->fileName() );
-				loaded << fi.fileName();
+			foreach(Iconset* tmp, PsiIconset::instance()->emoticons) {
+				loaded << tmp->id();
 			}
 		}
 
-		d->setCursor(WaitCursor);
+		d->setCursor(Qt::WaitCursor);
 
-		QPalette customPal = d->palette();
-		customPal.setDisabled(customPal.inactive());
 		d->ck_useEmoticons->setEnabled(false);
-		d->ck_useEmoticons->setPalette(customPal);
-
 		d->groupBox9->setEnabled(false);
-		d->groupBox9->setPalette(customPal);
 
 		d->progress->show();
-		d->progress->setProgress( 0 );
+		d->progress->setValue( 0 );
 
 		numIconsets = countIconsets("/emoticons", loaded);
 		iconsetsLoaded = 0;
@@ -543,7 +589,7 @@ bool OptionsTabIconsetEmoticons::event(QEvent *e)
 
 		if ( !numIconsets )
 			numIconsets = 1;
-		d->progress->setProgress( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
+		d->progress->setValue( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
 
 		Iconset *is = le->iconset();
 		if ( is ) {
@@ -556,12 +602,9 @@ bool OptionsTabIconsetEmoticons::event(QEvent *e)
 	}
 	else if ( e->type() == QEvent::User + 1 ) { // finish event
 		d->ck_useEmoticons->setEnabled(true);
-		d->ck_useEmoticons->unsetPalette();
-
 		d->groupBox9->setEnabled(true);
-		d->groupBox9->unsetPalette();
 
-		connect(d->iss_emoticons, SIGNAL(selectionChanged()), SIGNAL(dataChanged()));
+		connect(d->iss_emoticons, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SIGNAL(dataChanged()));
 
 		bool checked = d->ck_useEmoticons->isChecked();
 		d->ck_useEmoticons->setChecked( true );
@@ -582,12 +625,14 @@ void OptionsTabIconsetEmoticons::previewIconset()
 {
 	IconsetEmoUI *d = (IconsetEmoUI *)w;
 	const Iconset *is = d->iss_emoticons->iconset();
-	if ( is )
-		isDetails(*is, parentWidget);
+	if (is) {
+		isDetails(*is, parentWidget->parentWidget(), psi);
+	}
 }
 
-void OptionsTabIconsetEmoticons::setData(PsiCon *, QWidget *p)
+void OptionsTabIconsetEmoticons::setData(PsiCon *psicon, QWidget *p)
 {
+	psi = psicon;
 	parentWidget = p;
 }
 
@@ -607,10 +652,10 @@ void OptionsTabIconsetEmoticons::cancelThread()
 //----------------------------------------------------------------------------
 
 OptionsTabIconsetRoster::OptionsTabIconsetRoster(QObject *parent)
-: OptionsTab(parent, "iconset_roster", "", tr("Roster Iconsets"), tr("Select iconsets for your roster"), "psi/roster")
+	: OptionsTab(parent, "iconset_roster", "", tr("Roster Icons"), tr("Select iconsets for your roster"))
+	, w(0)
+	, thread(0)
 {
-	w = 0;
-	thread = 0;
 }
 
 OptionsTabIconsetRoster::~OptionsTabIconsetRoster()
@@ -618,6 +663,14 @@ OptionsTabIconsetRoster::~OptionsTabIconsetRoster()
 	cancelThread();
 }
 
+void OptionsTabIconsetRoster::addService(const QString& id, const QString& name)
+{
+	IconsetRosterUI*d = (IconsetRosterUI*)w;
+	QTreeWidgetItem* item = new QTreeWidgetItem(d->tw_isServices, QStringList(QString(name)));
+	item->setData(0, ServiceRole, QVariant(QString(id)));
+	d->tw_isServices->addTopLevelItem(item);
+}
+
 QWidget *OptionsTabIconsetRoster::widget()
 {
 	if ( w )
@@ -626,22 +679,34 @@ QWidget *OptionsTabIconsetRoster::widget()
 	w = new IconsetRosterUI;
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
 
+	// Initialize transport iconsets
+	addService("aim", "AIM");
+	addService("gadugadu", "Gadu-Gadu");
+	addService("icq", "ICQ");
+	addService("msn", "MSN");
+	addService("sms", "SMS");
+	addService("yahoo", "Yahoo!");
+	addService("transport", tr("Transport"));
+	d->tw_isServices->resizeColumnToContents(0);
+
 	connect(d->pb_defRosterDetails, SIGNAL(clicked()), SLOT(defaultDetails()));
 
-	connect(d->iss_servicesRoster, SIGNAL(selectionChanged(QListBoxItem *)), SLOT(isServices_iconsetSelected(QListBoxItem *)));
-	connect(d->lv_isServices, SIGNAL(selectionChanged(QListViewItem *)), SLOT(isServices_selectionChanged(QListViewItem *)));
+	connect(d->iss_servicesRoster, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)), SLOT(isServices_iconsetSelected(QListWidgetItem *, QListWidgetItem *)));
+	connect(d->tw_isServices, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)), SLOT(isServices_selectionChanged(QTreeWidgetItem *)));
 	connect(d->pb_servicesDetails, SIGNAL(clicked()), SLOT(servicesDetails()));
 	isServices_selectionChanged(0);
 
-	connect(d->iss_customRoster, SIGNAL(selectionChanged(QListBoxItem *)), SLOT(isCustom_iconsetSelected(QListBoxItem *)));
-	connect(d->lv_customRoster, SIGNAL(selectionChanged(QListViewItem *)), SLOT(isCustom_selectionChanged(QListViewItem *)));
+	connect(d->iss_customRoster, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)), SLOT(isCustom_iconsetSelected(QListWidgetItem *, QListWidgetItem *)));
+	connect(d->tw_customRoster, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)), SLOT(isCustom_selectionChanged(QTreeWidgetItem *)));
+	connect(d->tw_customRoster, SIGNAL(itemClicked(QTreeWidgetItem *, int )), SLOT(isCustom_selectionChanged(QTreeWidgetItem *)));
+
 	connect(d->pb_customRosterDetails, SIGNAL(clicked()), SLOT(customDetails()));
 	connect(d->le_customRoster, SIGNAL(textChanged(const QString &)), SLOT(isCustom_textChanged()));
 	connect(d->pb_customRosterAdd, SIGNAL(clicked()), SLOT(isCustom_add()));
 	connect(d->pb_customRosterDelete, SIGNAL(clicked()), SLOT(isCustom_delete()));
 	isCustom_selectionChanged(0);
 
-	QWhatsThis::add(d->ck_useTransportIconsForContacts,
+	d->ck_useTransportIconsForContacts->setWhatsThis(
 		tr("Toggles use of transport icons to the contacts, that use that transports."));
 
 	// TODO: add QWhatsThis
@@ -649,56 +714,59 @@ QWidget *OptionsTabIconsetRoster::widget()
 	return w;
 }
 
-void OptionsTabIconsetRoster::applyOptions(Options *opt)
+void OptionsTabIconsetRoster::applyOptions()
 {
-	if ( !w || thread )
+	if (!w || thread)
 		return;
 
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
-	opt->useTransportIconsForContacts = d->ck_useTransportIconsForContacts->isChecked();
+	PsiOptions::instance()->setOption("options.ui.contactlist.use-transport-icons", d->ck_useTransportIconsForContacts->isChecked());
 
 	// roster - default
 	{
 		const Iconset *is = d->iss_defRoster->iconset();
-		if ( is ) {
-			QFileInfo fi( is->fileName() );
-			opt->defaultRosterIconset = fi.fileName();
+		if (is) {
+			QFileInfo fi(is->fileName());
+			PsiOptions::instance()->setOption("options.iconsets.status", fi.fileName());
 		}
 	}
 
 	// roster - services
 	{
-		opt->serviceRosterIconset.clear();
+		QString baseServicesPath = "options.iconsets.service-status";
+		PsiOptions::instance()->removeOption(baseServicesPath, true);
 
-		QListViewItemIterator it( d->lv_isServices );
-		for ( ; it.current(); ++it) {
-			QListViewItem *item = it.current();
-
-			opt->serviceRosterIconset[item->text(2)] = item->text(3);
+		QTreeWidgetItemIterator it(d->tw_isServices);
+		while (*it) {
+			QString path = PsiOptions::instance()->mapPut(baseServicesPath, (*it)->data(0, ServiceRole).toString());
+			PsiOptions::instance()->setOption(path + ".iconset", (*it)->data(0, IconsetRole).toString());
+			++it;
 		}
 	}
 
 	// roster - custom
 	{
-		opt->customRosterIconset.clear();
+		QString baseCustomStatusPath = "options.iconsets.custom-status";
+		PsiOptions::instance()->removeOption(baseCustomStatusPath, true);
 
-		QListViewItemIterator it( d->lv_customRoster );
-		for ( ; it.current(); ++it) {
-			QListViewItem *item = it.current();
-
-			opt->customRosterIconset[item->text(2)] = item->text(3);
+		int index = 0;
+		QTreeWidgetItemIterator it(d->tw_customRoster);
+		while (*it) {
+			QString path = baseCustomStatusPath + ".a" + QString::number(index++);
+			PsiOptions::instance()->setOption(path + ".regexp", (*it)->data(0, RegexpRole).toString());
+			PsiOptions::instance()->setOption(path + ".iconset", (*it)->data(0, IconsetRole).toString());
+			++it;
 		}
 	}
 }
 
-void OptionsTabIconsetRoster::restoreOptions(const Options *opt)
+void OptionsTabIconsetRoster::restoreOptions()
 {
 	if ( !w || thread )
 		return;
 
-	o = (Options *)opt;
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
-	d->ck_useTransportIconsForContacts->setChecked( opt->useTransportIconsForContacts );
+	d->ck_useTransportIconsForContacts->setChecked( PsiOptions::instance()->getOption("options.ui.contactlist.use-transport-icons").toBool() );
 
 	d->iss_defRoster->clear();
 	d->iss_servicesRoster->clear();
@@ -706,15 +774,15 @@ void OptionsTabIconsetRoster::restoreOptions(const Options *opt)
 
 	QStringList loaded;
 
-	d->setCursor(WaitCursor);
+	d->setCursor(Qt::WaitCursor);
 
 	QPalette customPal = d->palette();
-	customPal.setDisabled(customPal.inactive());
+	// customPal.setDisabled(customPal.inactive());
 	d->tabWidget3->setEnabled(false);
 	d->tabWidget3->setPalette(customPal);
 
 	d->progress->show();
-	d->progress->setProgress( 0 );
+	d->progress->setValue( 0 );
 
 	numIconsets = countIconsets("/roster", loaded);
 	iconsetsLoaded = 0;
@@ -736,17 +804,17 @@ bool OptionsTabIconsetRoster::event(QEvent *e)
 
 		if ( !numIconsets )
 			numIconsets = 1;
-		d->progress->setProgress( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
+		d->progress->setValue( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
 
 		Iconset *i = le->iconset();
 		if ( i ) {
-			is->stripFirstAnimFrame(i);
+			PsiIconset::instance()->stripFirstAnimFrame(i);
 			QFileInfo fi( i->fileName() );
 
 			// roster - default
 			d->iss_defRoster->insert(*i);
-			if ( fi.fileName() == o->defaultRosterIconset )
-				d->iss_defRoster->setSelected(d->iss_defRoster->count()-1, true);
+			if ( fi.fileName() == PsiOptions::instance()->getOption("options.iconsets.status"))
+				d->iss_defRoster->setItemSelected(d->iss_defRoster->lastItem(), true);
 
 			// roster - service
 			d->iss_servicesRoster->insert(*i);
@@ -762,53 +830,56 @@ bool OptionsTabIconsetRoster::event(QEvent *e)
 	else if ( e->type() == QEvent::User + 1 ) { // finish event
 		// roster - service
 		{
-			// fill the QListView
-			QListViewItemIterator it( d->lv_isServices );
-			for ( ; it.current(); ++it) {
-				QListViewItem *i = it.current();
-				if ( !i->text(2).isEmpty() ) {
-					Iconset *iss = is->roster[o->serviceRosterIconset[i->text(2)]];
-					if ( iss ) {
+			// fill the QTreeWidget
+			QTreeWidgetItemIterator it(d->tw_isServices);
+			while (*it) {
+				QTreeWidgetItem *i = *it;
+				if (!i->data(0, ServiceRole).toString().isEmpty()) {
+					Iconset *iss = PsiIconset::instance()->roster[
+					                   PsiOptions::instance()->getOption(
+					                       PsiOptions::instance()->mapLookup(
+					                           "options.iconsets.service-status",
+					                           i->data(0, ServiceRole).toString())+".iconset").toString()];
+					if (iss) {
 						i->setText(1, iss->name());
-
-						QFileInfo fi ( iss->fileName() );
-						i->setText(3, fi.fileName());
+						QFileInfo fi(iss->fileName());
+						i->setData(0, IconsetRole, fi.fileName());
 					}
 				}
+				++it;
 			}
-
-			QHeader *head = d->lv_isServices->header();
-			head->removeLabel(2);
 		}
 
 		// roster - custom
 		{
 			// Then, fill the QListView
-			QListViewItem *last = 0;
-			QMap<QString, QString>::ConstIterator it = o->customRosterIconset.begin();
-			for ( ; it != o->customRosterIconset.end(); ++it) {
-				QListViewItem *item = new QListViewItem(d->lv_customRoster, last);
+			QTreeWidgetItem *last = 0;
+			QStringList customicons = PsiOptions::instance()->getChildOptionNames("options.iconsets.custom-status", true, true);
+			foreach(QString base, customicons) {
+				QString regexp = PsiOptions::instance()->getOption(base + ".regexp").toString();
+				QString icoset = PsiOptions::instance()->getOption(base + ".iconset").toString();
+				QTreeWidgetItem *item = new QTreeWidgetItem(d->tw_customRoster, last);
 				last = item;
 
-				item->setText(0, clipCustomText(it.key())); // RegExp
-				item->setText(2, it.key());
+				item->setText(0, clipCustomText(regexp));
+				item->setData(0, RegexpRole, regexp);
 
-				Iconset *iss = is->roster[it.data()];
+				Iconset *iss = PsiIconset::instance()->roster[icoset];
 				if ( iss ) {
 					item->setText(1, iss->name());
-
 					QFileInfo fi ( iss->fileName() );
-					item->setText(3, fi.fileName());
+					item->setData(0, IconsetRole, fi.fileName());
 				}
 			}
+			d->tw_customRoster->resizeColumnToContents(0);
 		}
 
 		d->tabWidget3->setEnabled(true);
-		d->tabWidget3->unsetPalette();
+		// d->tabWidget3->setPalette(QPalette());
 
-		connect(d->iss_defRoster, SIGNAL(selectionChanged()), SIGNAL(dataChanged()));
-		connect(d->iss_servicesRoster, SIGNAL(selectionChanged()), SIGNAL(dataChanged()));
-		connect(d->iss_customRoster, SIGNAL(selectionChanged()), SIGNAL(dataChanged()));
+		connect(d->iss_defRoster, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SIGNAL(dataChanged()));
+		connect(d->iss_servicesRoster, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SIGNAL(dataChanged()));
+		connect(d->iss_customRoster, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SIGNAL(dataChanged()));
 
 		d->progress->hide();
 
@@ -821,8 +892,9 @@ bool OptionsTabIconsetRoster::event(QEvent *e)
 	return false;
 }
 
-void OptionsTabIconsetRoster::setData(PsiCon *, QWidget *p)
+void OptionsTabIconsetRoster::setData(PsiCon *psicon, QWidget *p)
 {
+	psi = psicon;
 	parentWidget = p;
 }
 
@@ -830,35 +902,38 @@ void OptionsTabIconsetRoster::defaultDetails()
 {
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
 	const Iconset *is = d->iss_defRoster->iconset();
-	if ( is )
-		isDetails(*is, parentWidget);
+	if (is) {
+		isDetails(*is, parentWidget->parentWidget(), psi);
+	}
 }
 
 void OptionsTabIconsetRoster::servicesDetails()
 {
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
 	const Iconset *is = d->iss_servicesRoster->iconset();
-	if ( is )
-		isDetails(*is, parentWidget);
+	if (is) {
+		isDetails(*is, parentWidget->parentWidget(), psi);
+	}
 }
 
 void OptionsTabIconsetRoster::customDetails()
 {
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
 	const Iconset *is = d->iss_customRoster->iconset();
-	if ( is )
-		isDetails(*is, parentWidget);
+	if (is) {
+		isDetails(*is, parentWidget->parentWidget(), psi);
+	}
 }
 
 //------------------------------------------------------------
 
-void OptionsTabIconsetRoster::isServices_iconsetSelected(QListBoxItem *item)
+void OptionsTabIconsetRoster::isServices_iconsetSelected(QListWidgetItem *item, QListWidgetItem *)
 {
 	if ( !item )
 		return;
 
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
-	QListViewItem *it = d->lv_isServices->selectedItem();
+	QTreeWidgetItem *it = d->tw_isServices->currentItem();
 	if ( !it )
 		return;
 
@@ -868,10 +943,10 @@ void OptionsTabIconsetRoster::isServices_iconsetSelected(QListBoxItem *item)
 
 	it->setText(1, is->name());
 	QFileInfo fi ( is->fileName() );
-	it->setText(3, fi.fileName());
+	it->setData(0, IconsetRole, fi.fileName());
 }
 
-void OptionsTabIconsetRoster::isServices_selectionChanged(QListViewItem *it)
+void OptionsTabIconsetRoster::isServices_selectionChanged(QTreeWidgetItem *it)
 {
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
 	d->iss_servicesRoster->setEnabled( it != 0 );
@@ -880,21 +955,20 @@ void OptionsTabIconsetRoster::isServices_selectionChanged(QListViewItem *it)
 	if ( !it )
 		return;
 
-	if ( it->text(3).isEmpty() )
+	QString name = it->data(0, IconsetRole).toString();
+	if (name.isEmpty())
 		return;
 
-	QString name = it->text(3);
-
 	emit noDirty(true);
-	IconWidgetItem *item = (IconWidgetItem *)d->iss_servicesRoster->firstItem();
-	for ( ; item; item = (IconWidgetItem *)item->next()) {
+	for (int row = 0; row < d->iss_servicesRoster->count(); row++) {
+		IconWidgetItem *item = (IconWidgetItem *)d->iss_servicesRoster->item(row);
 		const Iconset *is = item->iconset();
 		if ( is ) {
 			QFileInfo fi ( is->fileName() );
 			if ( fi.fileName() == name ) {
 				emit noDirty(true);
-				d->iss_servicesRoster->setSelected(item, true);
-				d->iss_servicesRoster->ensureCurrentVisible();
+				d->iss_servicesRoster->setItemSelected(item, true);
+				d->iss_servicesRoster->scrollToItem(item);
 				emit noDirty(false);
 				break;
 			}
@@ -906,13 +980,13 @@ void OptionsTabIconsetRoster::isServices_selectionChanged(QListViewItem *it)
 
 //------------------------------------------------------------
 
-void OptionsTabIconsetRoster::isCustom_iconsetSelected(QListBoxItem *item)
+void OptionsTabIconsetRoster::isCustom_iconsetSelected(QListWidgetItem *item, QListWidgetItem *)
 {
 	if ( !item )
 		return;
 
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
-	QListViewItem *it = d->lv_customRoster->selectedItem();
+	QTreeWidgetItem *it = d->tw_customRoster->currentItem();
 	if ( !it )
 		return;
 
@@ -922,10 +996,10 @@ void OptionsTabIconsetRoster::isCustom_iconsetSelected(QListBoxItem *item)
 
 	it->setText(1, is->name());
 	QFileInfo fi ( is->fileName() );
-	it->setText(3, fi.fileName());
+	it->setData(0, IconsetRole, fi.fileName());
 }
 
-void OptionsTabIconsetRoster::isCustom_selectionChanged(QListViewItem *it)
+void OptionsTabIconsetRoster::isCustom_selectionChanged(QTreeWidgetItem *it)
 {
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
 	d->iss_customRoster->setEnabled( it != 0 );
@@ -937,21 +1011,21 @@ void OptionsTabIconsetRoster::isCustom_selectionChanged(QListViewItem *it)
 	if ( !it )
 		return;
 
-	if ( it->text(3).isEmpty() )
+	QString name = it->data(0, IconsetRole).toString();
+	if (name.isEmpty())
 		return;
 
 	emit noDirty(true);
-	d->le_customRoster->setText(it->text(2));
-	QString name = it->text(3);
+	d->le_customRoster->setText(it->data(0, RegexpRole).toString());
 
-	IconWidgetItem *item = (IconWidgetItem *)d->iss_customRoster->firstItem();
-	for ( ; item; item = (IconWidgetItem *)item->next()) {
+	for (int row = 0; row < d->iss_customRoster->count(); row++) {
+		IconWidgetItem *item = (IconWidgetItem *)d->iss_customRoster->item(row);
 		const Iconset *is = item->iconset();
 		if ( is ) {
 			QFileInfo fi ( is->fileName() );
 			if ( fi.fileName() == name ) {
-				d->iss_customRoster->setSelected(item, true);
-				d->iss_customRoster->ensureCurrentVisible();
+				d->iss_customRoster->setItemSelected(item, true);
+				d->iss_customRoster->scrollToItem(item);
 				break;
 			}
 		}
@@ -963,12 +1037,12 @@ void OptionsTabIconsetRoster::isCustom_selectionChanged(QListViewItem *it)
 void OptionsTabIconsetRoster::isCustom_textChanged()
 {
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
-	QListViewItem *item = d->lv_customRoster->selectedItem();
+	QTreeWidgetItem *item = d->tw_customRoster->currentItem();
 	if ( !item )
 		return;
 
-	item->setText( 0, clipCustomText(d->le_customRoster->text()) );
-	item->setText( 2, d->le_customRoster->text() );
+	item->setText(0, clipCustomText(d->le_customRoster->text()));
+	item->setData(0, RegexpRole, d->le_customRoster->text());
 }
 
 void OptionsTabIconsetRoster::isCustom_add()
@@ -977,23 +1051,23 @@ void OptionsTabIconsetRoster::isCustom_add()
 
 	QString def;
 	const Iconset *iconset = d->iss_defRoster->iconset();
-	if ( is ) {
+	if ( iconset ) {
 		QFileInfo fi( iconset->fileName() );
 		def = fi.fileName();
 	}
 	else
 		qWarning("OptionsTabIconsetRoster::isCustom_add(): 'def' is null!");
 
-	QListViewItem *item = new QListViewItem(d->lv_customRoster, d->lv_customRoster->lastItem());
-	const Iconset *i = is->roster[def];
+	QTreeWidgetItem *item = new QTreeWidgetItem(d->tw_customRoster);
+	const Iconset *i = PsiIconset::instance()->roster[def];
 	if ( i ) {
 		item->setText(1, i->name());
 
 		QFileInfo fi(i->fileName());
-		item->setText(3, fi.fileName());
+		item->setData(0, IconsetRole, fi.fileName());
 	}
 
-	d->lv_customRoster->setSelected(item, true);
+	d->tw_customRoster->setCurrentItem(item);
 	emit dataChanged();
 
 	d->le_customRoster->setFocus();
@@ -1002,12 +1076,13 @@ void OptionsTabIconsetRoster::isCustom_add()
 void OptionsTabIconsetRoster::isCustom_delete()
 {
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
-	QListViewItem *item = d->lv_customRoster->selectedItem();
+	QTreeWidgetItem *item = d->tw_customRoster->currentItem();
 	if ( !item )
 		return;
 
 	delete item;
 	isCustom_selectionChanged(0);
+	d->tw_customRoster->clearSelection();
 	emit dataChanged();
 }
 
