@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  */
 
@@ -32,7 +32,7 @@ class WinAmpController : public TuneController
 	Q_OBJECT
 public:
 	WinAmpController();
-	virtual Tune currentTune();
+	virtual Tune currentTune() const;
 
 protected:
 	QPair<bool, QString> getTrackTitle(HWND waWnd);
